/***************************************************************************
 *   Copyright (C) 2006 by Kevin Gilbert                                   *
 *   kev.gilbert@cdu.edu.au                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef _MAINWIDGET_H_
#define _MAINWIDGET_H_

#ifdef HAVE_CONFIG_H
#	include <config.h>
#endif

#include <qdatetime.h>
#include <qfont.h>
#include <qptrlist.h>
#include <qsplitter.h>
#include <kwallet.h>

#include "global.h"
#include "preferences.h"

using namespace KWallet;

//	forward class definitions
//	=========================

class CC_STATE;
class DisconnectedDlg;
class MainWidgetControl;
class MainWidgetView;
class QShowEvent;
class QTimer;
class QWidgetStack;
class RPC_CLIENT;

//	MainWidget class definition
//	===========================

class MainWidget : public QSplitter
{	Q_OBJECT

	public:
		MainWidget( QWidget* parent, const char* name );

		static MainWidget* self( ) { return m_self; }

		CPtr    connectToClient( const QString& clientName, const ushort portNo, const QString& rpcAuth );
		Wallet* openWallet( );
		void    readSettings( );
		void    saveSettings( );
		QString toolTipText( );
		void    wrapUp( );

		QPtrList<Client> getClients( ) { return MainWidget::self( )->m_clients; }

	signals:
		void changeCurrentClient( const QString& );
		void hasKonsole( const bool );
		void statusBarMessage( const QString& );
		void walletChanged( Wallet* );

	public slots:
		void slotConnectClient( );
		void slotDeleteClient( );
		void slotDisconnectClient( );
		void slotKonsoleClose( );
		void slotKonsoleOpen( );
		void slotPreferences( );
		void slotUpdateNow( );

	private slots:
		void slotApplyPreferences( const Preferences::PrefData* prefData );
		void slotChangeCurrentClient( const CPtr currentClient );
		void slotDisconnectedDlgCancelled( );
		void slotRetryConnection( const CPtr client );
		void slotUpdate( );
		void slotUpdateMessage( );

	private:
		void closeWallet( );
		bool connectToClient( const CPtr client );
		void connectToClientFailed( const CPtr client );
		bool createDisconnectedClientDlg( );
		CPtr disconnectClient( const QString& clientName );
		void disconnectClient( const CPtr client );
		void firstUpdate( );
		void removeDisconnectedClient( const CPtr client );
		bool retryConnection( const CPtr client, const time_t now );
		void showEvent( QShowEvent* event );
		void update( );
		void updateDisconnectedClientStatus( const CPtr client, const QString& text );
		void updateDisconnectedClientRetryCount( const CPtr client );
		void updateProjects( );

		static MainWidget* m_self;

		QPtrList<Client>   m_clients;
		CPtr               m_currentClient;
		MainWidgetControl* m_controlView;
		bool               m_disconnectedCancelled;
		DisconnectedDlg*   m_disconnectedDlg;
		QFont              m_font;
		bool               m_firstUpdate;
		QTimer*            m_messageTimer;
		QDateTime          m_projectTimeNext;
		uint               m_projectUpdateInterval;
		uint               m_projectUpdateTOD;
		bool               m_retryExpBack;
		uint               m_retryExpBackLimit;
		int                m_retryLimit;
		bool               m_retryLostCxns;
		QWidgetStack*      m_stackView;
		bool               m_synchroniseLayouts;
		uint               m_updateInterval;
		QDateTime          m_updateTimeNext;
		QDateTime          m_updateTimePrior;
		QTimer*            m_updateTimer;
		Wallet*            m_wallet;
		QString            m_walletName;
};

#endif	// _MAINWIDGET_H_
